# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import base64
from Crypto.Cipher import AES

from django.middleware.csrf import get_token
from django.utils.functional import lazy

from debreach.compat import \
    get_random_string, text_type, force_bytes, force_text


def csrf(request):
    """
    Context processor that provides a CSRF token, or the string 'NOTPROVIDED'
    if it has not been provided by either a view decorator or the middleware
    """
    def _get_val():
        token = get_token(request)
        if token is None:
            # In order to be able to provide debugging info in the
            # case of misconfiguration, we use a sentinel value
            # instead of returning an empty dict.
            return 'NOTPROVIDED'
        else:
            key = force_bytes(get_random_string(16))
            aes = AES.new(key)
            pad_length = 16 - (len(token) % 16 or 16)
            padding = ''.join('#' for _ in range(pad_length))
            value = base64.b64encode(
                aes.encrypt('{0}{1}'.format(token, padding))
            )
            token = '$'.join((force_text(key), force_text(value)))
            return force_text(token)
    _get_val = lazy(_get_val, text_type)

    return {'csrf_token': _get_val()}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import base64
import logging
import random
from Crypto.Cipher import AES

from django.core.exceptions import SuspiciousOperation

from debreach.compat import \
    force_bytes, get_random_string, string_types, binary_type, force_text


log = logging.getLogger(__name__)


class CSRFCryptMiddleware(object):

    def process_request(self, request):
        if request.POST.get('csrfmiddlewaretoken') \
                and '$' in request.POST.get('csrfmiddlewaretoken'):
            try:
                post_was_mutable = request.POST._mutable
                POST = request.POST.copy()
                token = POST.get('csrfmiddlewaretoken')
                key, value = token.split('$')
                aes = AES.new(key)
                POST['csrfmiddlewaretoken'] = force_bytes(
                    aes.decrypt(
                        force_bytes(base64.b64decode(value))).rstrip(b'#')
                )
                POST._mutable = post_was_mutable
                request.POST = POST
            except:
                log.exception('Error decoding csrfmiddlewaretoken')
                raise SuspiciousOperation(
                    'csrfmiddlewaretoken has been tampered with')
        if request.META.get('HTTP_X_CSRFTOKEN') \
                and '$' in request.META.get('HTTP_X_CSRFTOKEN'):
            try:
                META = request.META.copy()
                token = META.get('HTTP_X_CSRFTOKEN')
                key, value = token.split('$')
                aes = AES.new(key)
                META['HTTP_X_CSRFTOKEN'] = force_bytes(
                    aes.decrypt(base64.b64decode(value)).rstrip(b'#')
                )
                request.META = META
            except:
                log.exception('Error decoding csrfmiddlewaretoken')
                raise SuspiciousOperation(
                    'X-CSRFToken header has been tampered with')
            return


class RandomCommentMiddleware(object):

    def process_response(self, request, response):
        str_types = string_types + (binary_type,)
        if not getattr(response, 'streaming', False) \
                and response['Content-Type'].startswith('text/html') \
                and isinstance(response.content, str_types):
            comment = '<!-- {0} -->'.format(
                get_random_string(random.choice(range(12, 25))))
            response.content = '{0}{1}'.format(
                force_text(response.content), comment)
        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import re

from django.core.exceptions import SuspiciousOperation
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.test import TestCase
from django.test.client import RequestFactory
from django.utils.crypto import get_random_string
from django.utils.unittest import skipUnless

from debreach.compat import force_text
from debreach.context_processors import csrf
from debreach.middleware import CSRFCryptMiddleware, RandomCommentMiddleware

try:
    unichr
except NameError:
    pass
else:
    chr = unichr


class TestCSRFCryptMiddleware(TestCase):

    def test_not_encoded(self):
        request = RequestFactory().post('/', {'csrfmiddlewaretoken': 'abc123'})
        middleware = CSRFCryptMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.POST.get('csrfmiddlewaretoken'), 'abc123')

    def test_encoded(self):
        request = RequestFactory().post(
            '/',
            {'csrfmiddlewaretoken':
                'WaMeyTIUS6hOoTcm$TOKqMT3J0Gx2b15UH1MkRg=='}
        )
        middleware = CSRFCryptMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.POST.get('csrfmiddlewaretoken'), 'abc123')

    def test_mutable_status(self):
        request = RequestFactory().post(
            '/',
            {'csrfmiddlewaretoken':
                'WaMeyTIUS6hOoTcm$TOKqMT3J0Gx2b15UH1MkRg=='}
        )
        request.POST._mutable = False
        middleware = CSRFCryptMiddleware()
        middleware.process_request(request)
        self.assertFalse(request.POST._mutable)
        request = RequestFactory().post(
            '/',
            {'csrfmiddlewaretoken':
                'WaMeyTIUS6hOoTcm$TOKqMT3J0Gx2b15UH1MkRg=='}
        )
        request.POST._mutable = True
        middleware = CSRFCryptMiddleware()
        middleware.process_request(request)
        self.assertTrue(request.POST._mutable)

    def test_header_not_encoded(self):
        request = RequestFactory().post('/', HTTP_X_CSRFTOKEN='abc123')
        middleware = CSRFCryptMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.META.get('HTTP_X_CSRFTOKEN'), 'abc123')

    def test_header_encoded(self):
        request = RequestFactory().post(
            '/', HTTP_X_CSRFTOKEN='WaMeyTIUS6hOoTcm$TOKqMT3J0Gx2b15UH1MkRg==',
        )
        middleware = CSRFCryptMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.META.get('HTTP_X_CSRFTOKEN'), b'abc123')

    def test_tampering(self):
        request = RequestFactory().post(
            '/', {'csrfmiddlewaretoken': '123$abc'})
        middleware = CSRFCryptMiddleware()
        with self.assertRaises(SuspiciousOperation):
            middleware.process_request(request)

    def test_header_tampering(self):
        request = RequestFactory().post('/', HTTP_X_CSRFTOKEN='123$abc')
        middleware = CSRFCryptMiddleware()
        with self.assertRaises(SuspiciousOperation):
            middleware.process_request(request)


class TestRandomCommentMiddleware(TestCase):

    def test_noop_on_wrong_content_type(self):
        response = HttpResponse('abc', content_type='text/plain')
        request = RequestFactory().get('/')
        middleware = RandomCommentMiddleware()
        response = middleware.process_response(request, response)
        self.assertEqual(response.content, b'abc')

    def test_html_content_type(self):
        html = '''<!doctype html>
<html>
    <head>
        <title>Page title</title>
    </head>
    <body>
        <h1>Test</h1>
        <p>Lorem ipsum</p>
    </body>
</html>'''
        response = HttpResponse(html, content_type='text/html')
        request = RequestFactory().get('/')
        middleware = RandomCommentMiddleware()
        response = middleware.process_response(request, response)
        self.assertNotEqual(response.content, html)

    def test_unicode_characters(self):
        html = '''<!doctype html>
<html>
    <head>
        <title>Page title</title>
    </head>
    <body>
        <h1>Test</h1>
        <p>{0}</p>
    </body>
</html>'''.format(''.join(chr(x) for x in range(9999)))
        response = HttpResponse(html, content_type='text/html')
        request = RequestFactory().get('/')
        middleware = RandomCommentMiddleware()
        response = middleware.process_response(request, response)
        self.assertNotEqual(force_text(response.content), force_text(html))


class TestContextProcessor(TestCase):

    def test_csrf(self):
        request = RequestFactory().get('/')
        request.META['CSRF_COOKIE'] = 'abc123'
        context = csrf(request)
        self.assertTrue(force_text(context['csrf_token']))
        self.assertNotEqual(force_text(context['csrf_token']), 'abc123')

    def test_no_token_csrf(self):
        request = RequestFactory().get('/')
        context = csrf(request)
        self.assertTrue(force_text(context['csrf_token']))
        self.assertEqual(force_text(context['csrf_token']), 'NOTPROVIDED')


@skipUnless(
    'test_project' in os.environ.get('DJANGO_SETTINGS_MODULE', ''),
    'Not running in test_project'
)
class IntegrationTests(TestCase):

    def test_adds_comment(self):
        resp = self.client.get(reverse('home'))
        self.assertFalse(resp.content.endswith(b'</html>'))

    def test_crypt_csrf_token(self):
        resp = self.client.get(reverse('test_form'))
        m = re.search(
            r'value=\'(.*\$.*)\'',
            force_text(resp.content),
            re.MULTILINE | re.DOTALL
        )
        self.assertEqual(len(m.groups()), 1)
        token = m.groups()[0].strip()
        post_resp = self.client.post(
            reverse('test_form'),
            {'csrfmiddlewaretoken': token, 'message': 'Some rubbish'}
        )
        self.assertRedirects(post_resp, reverse('home'))

    def test_crypt_csrf_header(self):
        resp = self.client.get(reverse('test_form'))
        m = re.search(
            r'value=\'(.*\$.*)\'',
            force_text(resp.content),
            re.MULTILINE | re.DOTALL
        )
        self.assertEqual(len(m.groups()), 1)
        token = m.groups()[0].strip()
        post_resp = self.client.post(
            reverse('test_form'),
            {'message': 'Some rubbish'},
            X_CSRFTOKEN=token,
        )
        self.assertRedirects(post_resp, reverse('home'))

    def test_round_trip_loop(self):
        '''
        Checks a wide range of input tokens and keys
        '''
        for _ in range(1000):
            request = RequestFactory().get('/')
            csrf_token = get_random_string(32)
            request.META['CSRF_COOKIE'] = csrf_token
            token = force_text(csrf(request)['csrf_token'])
            request = RequestFactory().post(
                '/', {'csrfmiddlewaretoken': token})
            middleware = CSRFCryptMiddleware()
            middleware.process_request(request)
            self.assertEqual(
                force_text(request.POST.get('csrfmiddlewaretoken')),
                force_text(csrf_token)
            )

    def test_round_trip_loop_header(self):
        '''
        Checks a wide range of input tokens and keys
        '''
        for _ in range(1000):
            request = RequestFactory().get('/')
            csrf_token = get_random_string(32)
            request.META['CSRF_COOKIE'] = csrf_token
            token = csrf(request)['csrf_token']
            request = RequestFactory().post(
                '/',
                HTTP_X_CSRFTOKEN=force_text(token),
                HTTP_X_REQUESTED_WITH='XMLHttpRequest'
            )
            middleware = CSRFCryptMiddleware()
            middleware.process_request(request)
            self.assertEqual(
                force_text(request.META.get('HTTP_X_CSRFTOKEN')),
                force_text(csrf_token)
            )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


try:
    # Django >= 1.4.5
    from django.utils.encoding import force_bytes, force_text, smart_text  # NOQA
    from django.utils.six import string_types, text_type, binary_type  # NOQA
except ImportError:  # pragma: no cover
    # Django < 1.4.5
    from django.utils.encoding import (                         # NOQA
        smart_unicode as smart_text, smart_str as force_bytes)  # NOQA
    force_text = smart_text
    string_types = (basestring,)
    text_type = unicode
    binary_type = str

try:
    # Django >= 1.4
    from django.utils.crypto import get_random_string  # NOQA
except ImportError:  # pragma: no cover
    # Django < 1.4
    from random import choice
    get_random_string = lambda n: ''.join(
        [choice('abcdefghijklmnopqrstuvwxyz0123456789') for i in range(n)])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import sys
from setuptools import setup, find_packages


def get_version(package):
    '''
    Return package version as listed in `__version__` in `init.py`.
    '''
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search(
        '^__version__ = [\'"]([^\'"]+)[\'"]', init_py, re.MULTILINE
    ).group(1)


version = get_version('debreach')


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    args = {'version': version}
    print('You probably want to also tag the version now:')
    print(' git tag -a release/{version} -m \'version {version}\''.format(
        **args))
    print(' git push --tags')
    sys.exit()


setup(
    name='django-debreach',
    version=version,
    url='http://github.com/lpomfrey/django-debreach',
    license='BSD',
    description='Adds middleware and context processors to give some '
                'protection against the BREACH attack in Django.',
    author='Luke Pomfrey',
    author_email='lpomfrey@gmail.com',
    packages=find_packages(),
    install_requires=open('requirements.txt').read().split('\n'),
    test_suite='runtests.runtests',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP'
    ]
)
